from abc import ABC, abstractmethod

class Intranet(ABC):
    """Interface of the communication between model and intranet

    :param token: Must be set before anything
    :type token: str
    """

    @abstractmethod
    async def get_current_scholar_year(self):
        """Get current Scolar year from the intranet
        """
        ...
    
    @abstractmethod
    async def get_students(self, promotions: str):
        """Get list of students of the given promotions

        :param promotions: Promotions code (Defined in constants.py)
        :type promotion: str
        :returns: Students or None in case of error
        :rtype: typing.Optional[list[str]]
        """
        ...

    @abstractmethod
    async def register_students(self, event: str, students: list[str]):
        """Register students to given event

        :param event: Epitech intranet url (format: 'event-xxxxxx')
        :type event: str
        :param students: Students to register.
        :type students: list[str]
        :returns: None on failure, the list of students registered on success
        :rtype: typing.Optional(list[str])
        """
        ...

    @abstractmethod
    async def create_event(self, activity: str, date: str, hour: str):
        """Create event at a given date for a given activity

        :param activity: Activity url under which to create event
        :type activity: str
        :param date: Date of the event to create (format 'YYYY-MM-DD')
        :type date: str
        :param hour: hour of the start of the event to create (format 'HH-MM-SS')
        :type hour: str
        :returns: A code of the created event
        :rtype: str
        """
        ...

    @abstractmethod
    async def get_events(self, activity: str, *, date: str = None, hour: str = None):
        """Get All events for a given activity
        
        :param activity: Activity url
        :type activity: str
        :param date: date of event to get (format: YYYY-MM-DD)
        :type date: typing.Optional(str)
        :param hour: hour of event to get (format: 00:00)
        :returns: Events code of the activity
        :rtype: list
        """
        ...

    @abstractmethod
    async def get_registered_students(self, event: str):
        """Get All students registered to the given event
        
        :param event: Event url
        :type event: str
        :returns: Registered students
        :rtye: list[dict]
        """
        ...

    @abstractmethod
    async def get_activities(self, year: str, codeModule: str, codeInstance: str):
        """Get Activities in a module given a code and an instance

        :param year: accademic year
        :type year: str
        :param codeModule: Code of the module
        :type codeModule: str
        :param codeInstance: Year instance (eg: LYN-0-1)
        :type codeInstance: str
        """
        ...
    
    @abstractmethod
    async def get_modules(self, year: str):
        """Get All modules for a year

        :param year: Accademic year
        :type year: str
        """
        ...
    
    @abstractmethod
    async def get_module(self, year: str, code: str, instance: str):
        """Get one module

        :param year: Accademic year
        :type year: str
        :param code: module code
        :type code: str
        :param instance: module instance
        :type instance: str
        """
        ...


import re
from .constants import INTRA_LINK

def execRegex(regex, string):
    """Check if regex match in string

    :params regex: A regex
    :type regex: str
    :params string: A string to match the *regex*
    :returns: A boolean, True if *regex* is found in *string* False otherwise
    :rtype: bool
    """
    matches = re.finditer(regex, string, re.MULTILINE)
    for match in matches:
        if match:
            return True
    return False

def check_autologin(autologin):
    """Check Autologin format
    
    :param autologin: An Epitech Intranet autologin url (found under the administration tab on the Epitech intranet) 
    :type autologin: str
    :returns: A correctly formated autologin url if the format is correct, False otherwise
    :rtype: str
    """
    if execRegex(r'^http://', autologin):
        autologin = autologin.replace('http', 'https')
    if execRegex(rf'^{INTRA_LINK}/auth-[0-9a-fA-F]+$', autologin):
        autologin = 'https://' + autologin
        return autologin
    elif execRegex(r'^auth-[0-9a-fA-F]+$', autologin):
        return f'https://intra.epitech.eu/' + autologin
    elif execRegex(rf'^https://{INTRA_LINK}/auth-[0-9a-fA-F]+$', autologin):
        return autologin
    return False



PROMOTIONS = {
    'msc1': 'msc4',
    'msc2': 'msc5',
    'premsc': 'msc3',
    'wac1': 'wac1',
    'wac2': 'wac2',
    'tek1': 'tek1',
    'tek2': 'tek2',
    'tek3': 'tek3',
}

courses = 'course[]=Code-And-Go&course[]=Dev-And-Go&course[]=msc&course[]=premsc&course[]=webacademie&course[]=other'

INTRA_LINK = r'intra\.epitech\.eu'



import requests

from time import sleep
import re
import json
from .abc import Intranet
from .checkers import check_autologin
from .constants import PROMOTIONS, courses

class AutologinIntranet(Intranet):
    def __init__(self, token=""):
        self.token = check_autologin(token)

    def set_token(self, token):
        self.token = check_autologin(token)

    def get_current_scholar_year(self):
        content = requests.get(
            f'https://intra.epitech.eu/').content.decode('utf-8')
        matches = re.finditer(r'\"scolaryear\":([0-9]{4})', content)
        result = filter(id, matches)
        return next(result)[1]
 
    def get_students(self, promotion, year):
        results, nb_items, total = [], 0, 1
        promo = f'{PROMOTIONS[promotion]}'
        while nb_items < total:
            try:
                req = requests.get(
                    f'{self.token}/user/filter/user?format=json&location=FR/LYN&year={year}&active=true&promo={promo}&offset={nb_items}'
                )
            except Exception as e:
                print(f'[getStudents] An error occured while asking intra : {e}')
                return None
            results += [elem['login'] for elem in req.json()['items']]
            total = req.json()['total']
            nb_items += len(req.json()['items'])
        return results
    
    def register_students(self, event, students):
        if not students:
            return None
        data = {
            f'items[{index}][login]' : row
            for index, row in enumerate(students)
        }
        try:
            req = requests.post(
                f'{self.token}{event}/updateregistered?format=json', data)
        except Exception as e:
            print(f'[registerStudents] An error occured while asking intra : {e}')
            return None
        sleep(0.2) # HACK: avoid intra rate limit
        return students

    def create_event(self, activity, date, hour):
        acti_data = {'start': date + ' ' + hour}
        try:
            req = requests.post(
                f'{self.token}{activity}planify?format=json', acti_data)
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
            return None
        sleep(0.2) # HACK: avoid intra rate limit
        return req.json()

    def get_events(self, activity, *, date=None, hour=None):
        try:
            activities = requests.get(f'{self.token}{activity}?format=json')
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
            return None
        activities_json = activities.json()
        events = activities_json.get('events', None)
        if events == None:
            error_message = activities_json.get('message', None)
            print('An error occured with the request : unable to get content {error_message}')
            return None
        if date and hour:
            today_event = [
                a['code'] 
                for a in events
                if a['begin'][:16] == f'{date} {hour}'
            ]
            return today_event
        elif date:
            today_event = [
                a['code'] 
                for a in events
                if a['begin'][:10] == date
            ]
            return today_event
        elif hour:
            today_event = [
                a['code'] 
                for a in events
                if a['begin'][11:16] == hour
            ]
            return today_event
        return events

    def get_registered_students(self, event):
        try:
            students = requests.get(
                f'{self.token}{event}/registered?format=json')
            presence = {
                a['login'] : a['present']
                for a in students.json()
            }
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
        return presence
    
    def get_activities(self, year, codeModule, codeInstance):
        try:
            data = requests.get(
                f'{self.token}/module/{year}'
                f'/{codeModule}/{codeInstance}/?format=json')
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
            return
        activities = data.json()
        return activities
    
    def get_modules(self, year):
        try:
            data = requests.get(
                f'{self.token}/course/filter'
                f'?format=json&preload=1&scolaryear[]={year}&{courses}')
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
            return

        result = data.json()
        return result['items']
    
    def get_module(self, year, code, instance):
        try:
            data = requests.get(
                f'{self.token}/module/{year}/{code}/{instance}/?format=json')
        except Exception as e:
            print(f'An error occured while asking intra : {e}')
            return
        result = data.json()
        return result